/* gtkwsigmeter.c
 * Copyright (C) 2002 Todd Fisher
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtkmain.h>
#include <gtk/gtksignal.h>
#include <iwlib.h>
#include <math.h>
#include <string.h>
#include "gtkwsigmeter.h"

#define WSIGMETER_DEFAULT_HEIGHT 120
#define WSIGMETER_DEFAULT_WIDTH 280
#define WSIGMETER_UDPATE_INTERVAL 500
/* Forward declarations */
static void
gtk_wsigmeter_class_init (GtkwSigMeterClass *class);
static void
gtk_wsigmeter_init( GtkwSigMeter *sig );
static void
gtk_wsigmeter_destroy( GtkObject *object );
static void
gtk_wsigmeter_realize( GtkWidget *widget );
static void
gtk_wsigmeter_size_request( GtkWidget *widget,
														GtkRequisition *requisition );
static gboolean
gtk_wsigmeter_update( GtkWidget *widget );

static void 
gtk_wsigmeter_size_allocate( GtkWidget *widget,
														 GtkAllocation *alloc );
static gint
gtk_wsigmeter_configure( GtkWidget *widget,
												 GdkEventConfigure *event );
static gint
gtk_wsigmeter_expose( GtkWidget *widget,
											GdkEventExpose *event );

/* Local data */
static GtkWidgetClass *parent_class = NULL;

GtkType
gtk_wsigmeter_get_type()
{
	static GtkType wsigmeter_type = 0;
	if( !wsigmeter_type ){
		static const GtkTypeInfo wsigmeter_info = 
		{
			"GtkwSigMeter",
			sizeof (GtkwSigMeter),
			sizeof (GtkwSigMeterClass),
			(GtkClassInitFunc)gtk_wsigmeter_class_init,
			(GtkObjectInitFunc)gtk_wsigmeter_init,
			/* reserved_1 */ NULL,
			/* reserved_1 */ NULL,
			(GtkClassInitFunc) NULL
		};
		wsigmeter_type = gtk_type_unique( GTK_TYPE_WIDGET, &wsigmeter_info );
	}
	return wsigmeter_type;
}
static void
gtk_wsigmeter_class_init (GtkwSigMeterClass *class)
{
	GtkObjectClass *object_class;
	GtkWidgetClass *widget_class;

	object_class = (GtkObjectClass*)class;
	widget_class = (GtkWidgetClass*)class;

	parent_class = gtk_type_class( gtk_widget_get_type() );

	object_class->destroy 				= gtk_wsigmeter_destroy;
	widget_class->realize 				= gtk_wsigmeter_realize;
	widget_class->configure_event = gtk_wsigmeter_configure;
	widget_class->expose_event 		= gtk_wsigmeter_expose;
	widget_class->size_request 		= gtk_wsigmeter_size_request;
	widget_class->size_allocate 	= gtk_wsigmeter_size_allocate;
/*	widget_class->button_press_event = gtk_wsigmeter_button_press;
	widget_class->button_release_event = gtk_wsigmeter_button_release;
	widget_class->motion_notify_event = gtk_wsigmeter_motion_notify;*/
}
static void
gtk_wsigmeter_init( GtkwSigMeter *sig )
{
	sig->link_quality         = 0;
	sig->max_link_quality     = 0;
	sig->sig_level            = 0;
	sig->max_sig_level        = 0;
	sig->noise_level          = 0;
	sig->max_noise_level      = 0;
	sig->freq                 = 0;
	sig->mode                 = 0;
	sig->link_qual_text_on    = TRUE;
	sig->link_qual_meter_on   = TRUE;
	sig->sig_level_text_on    = TRUE;
	sig->sig_level_meter_on   = TRUE;
	sig->noise_level_text_on  = TRUE;
	sig->noise_level_meter_on = TRUE;
	sig->network_info_on      = TRUE;
	sig->pixmap               = NULL;
	sig->layout               = NULL;

	memset( sig->network_id, 0, 256 );
	memset( sig->nickname, 0, 256 );
	memset( sig->hwaddr, 0, 256 );

	/* create the pango layout for drawing text */
	sig->layout = pango_layout_new( gdk_pango_context_get() );
	g_return_if_fail( sig->layout != NULL );
	/* open socket to wireless device */
	sig->skfd = iw_sockets_open();

	sig->device_name = g_strdup( "eth0" );
}
GtkWidget*
gtk_wsigmeter_new(void)
{
	GtkwSigMeter *sig;
	sig = gtk_type_new( gtk_wsigmeter_get_type() );
	return GTK_WIDGET(sig);
}
static void
gtk_wsigmeter_destroy( GtkObject *object )
{
	GtkwSigMeter *sig = GTK_WSIGMETER(object);

	g_return_if_fail( object != NULL );
	g_return_if_fail( GTK_IS_WSIGMETER(object) );
	
	/* first thing to do is stop the timer*/
	g_idle_remove_by_data( &(sig->timer) );
	
	if( sig->pixmap != NULL ){
		g_object_unref( sig->pixmap );
	//	g_free( sig->pixmap );
		sig->pixmap = NULL;
	}
	if( sig->layout != NULL ){
		g_object_unref( sig->layout );
	//	g_free( sig->layout );
		sig->layout = NULL;
	}
	if( GTK_OBJECT_CLASS(parent_class)->destroy ){
		(*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
	}
	close( sig->skfd );
	if( sig->device_name )
		g_free( sig->device_name );
}
gint 
gtk_wsigmeter_get_link_quality( GtkwSigMeter* s, gint *max )
{
	if( max )
		*max = s->max_link_quality;
	return s->link_quality;
}
gint 
gtk_wsigmeter_get_sig_level( GtkwSigMeter* s, gint *max )
{
	if( max )
		*max = s->max_sig_level;
	return s->sig_level;
}
gint 
gtk_wsigmeter_get_noise_level( GtkwSigMeter* s, gint *max )
{
	if( max )
		*max = s->max_noise_level;
	return s->noise_level;
}
gint 
gtk_wsigmeter_get_freq( GtkwSigMeter* s )
{
	return s->freq;
}
GtkWirelessMode 
gtk_wsigmeter_get_mode( GtkwSigMeter* s )
{
	return s->mode;
}
const gchar* 
gtk_wsigmeter_get_network_id( GtkwSigMeter* s )
{
	return s->network_id;
}
const gchar*
gtk_wsigmeter_get_protocol( GtkwSigMeter* s )
{
	return s->protocol;
}
const gchar* 
gtk_wsigmeter_get_nickname( GtkwSigMeter* s )
{
	return s->nickname;
}
const gchar* gtk_wsigmeter_get_device_name( GtkwSigMeter* s )
{
	return s->device_name;
}
void gtk_wsigmeter_set_device_name( GtkwSigMeter*s, const gchar *name )
{
	if( s->device_name )
		g_free( s->device_name );
	s->device_name = g_strdup( name );
}
const gchar* 
gtk_wsigmeter_get_hwaddr( GtkwSigMeter* s )
{
	return s->hwaddr;
}
gint
gtk_wsigmeter_get_update_interval( GtkwSigMeter*s )
{
	return s->update_interval;
}
void 
gtk_wsigmeter_set_update_interval( GtkwSigMeter*s, gint update_interval )
{
	s->update_interval = update_interval;
	g_source_remove( s->timer );
	
	if( s->update_interval > 20 ){
		s->timer = g_timeout_add( s->update_interval, 
														 (GSourceFunc)gtk_wsigmeter_update, 
										         (gpointer)s ); 
	}
}
void 
gtk_wsigmeter_toggle_link_qual_text( GtkwSigMeter* s, gboolean on )
{
	s->link_qual_text_on = on;
	gtk_widget_queue_draw( GTK_WIDGET( s ) );
}
void 
gtk_wsigmeter_toggle_link_qual_meter( GtkwSigMeter* s, gboolean on )
{
	s->link_qual_meter_on = on;
	gtk_widget_queue_draw( GTK_WIDGET( s ) );
}
void 
gtk_wsigmeter_toggle_signal_level_text( GtkwSigMeter* s, gboolean on )
{
	s->sig_level_text_on = on;
	gtk_widget_queue_draw( GTK_WIDGET( s ) );
}
void 
gtk_wsigmeter_toggle_signal_level_meter( GtkwSigMeter* s, gboolean on )
{
	s->sig_level_meter_on = on;
	gtk_widget_queue_draw( GTK_WIDGET( s ) );
}
void 
gtk_wsigmeter_toggle_noise_level_text( GtkwSigMeter* s, gboolean on )
{
	s->noise_level_text_on = on;
	gtk_widget_queue_draw( GTK_WIDGET( s ) );
}
void 
gtk_wsigmeter_toggle_noise_level_meter( GtkwSigMeter* s, gboolean on )
{
	s->noise_level_meter_on = on;
	gtk_widget_queue_draw( GTK_WIDGET( s ) );
}

void
gtk_wsigmeter_toggle_network_info( GtkwSigMeter* s, 
															 		 gboolean on )
{
	s->network_info_on = on;
}
static void
gtk_wsigmeter_realize( GtkWidget *widget )
{
	GtkwSigMeter *sig;
	GdkWindowAttr attr;
	gint attributes_mask;

	g_return_if_fail( widget != NULL );
	g_return_if_fail( GTK_IS_WSIGMETER(widget) );

	GTK_WIDGET_SET_FLAGS( widget, GTK_REALIZED );
	sig = GTK_WSIGMETER( widget );

	attr.x 						= widget->allocation.x;
	attr.y 						= widget->allocation.y;
	attr.width 				= widget->allocation.width;
	attr.height				= widget->allocation.height;
	attr.wclass 			= GDK_INPUT_OUTPUT;
	attr.window_type	= GDK_WINDOW_CHILD;
	attr.event_mask 	= gtk_widget_get_events( widget ) | GDK_EXPOSURE_MASK | GDK_STRUCTURE_MASK;
	attr.visual 			= gtk_widget_get_visual( widget );
	attr.colormap 		= gtk_widget_get_colormap( widget );
	attributes_mask 	= GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;

	widget->window = gdk_window_new( widget->parent->window, &attr, attributes_mask );

	widget->style = gtk_style_attach( widget->style, widget->window );

	gdk_window_set_user_data( widget->window, widget );

	gtk_style_set_background( widget->style, widget->window, GTK_STATE_ACTIVE );
#if 0
	/* create the pango layout for drawing text */
	sig->layout = pango_layout_new( gdk_pango_context_get() );
	g_return_if_fail( sig->layout != NULL );
	/* open socket to wireless device */
	sig->skfd = iw_sockets_open();
#endif
	gtk_wsigmeter_configure( widget, NULL );
	/* set time out function to be called at a predefined interval */
	sig->update_interval = WSIGMETER_UDPATE_INTERVAL;
	sig->timer = g_timeout_add( sig->update_interval, 
			      								(GSourceFunc)gtk_wsigmeter_update, 
							              (gpointer)widget );
}
static gint 
calc_size( GtkwSigMeter *sig, 
					 gint y, 
					 gint offset,
					 gboolean on )
{
	const char *sample_text = "Sample Text jI";
	gint text_width, text_height;
	if( on ){
		pango_layout_set_text( sig->layout, sample_text, strlen( sample_text ) );
		pango_layout_get_pixel_size( sig->layout, &text_width, &text_height );
		y += text_height + offset;
	}
	return y;
}
static void
gtk_wsigmeter_size_request( GtkWidget *widget,
														GtkRequisition *req )
{
	req->width = WSIGMETER_DEFAULT_WIDTH;
	if( GTK_WSIGMETER(widget)->layout != NULL ){
		req->height =  calc_size( GTK_WSIGMETER(widget), 0, 10, 
														  GTK_WSIGMETER(widget)->network_info_on );
		req->height += calc_size( GTK_WSIGMETER(widget), 0, 10, 
															GTK_WSIGMETER(widget)->link_qual_meter_on | 
															GTK_WSIGMETER(widget)->link_qual_text_on );
		req->height += calc_size( GTK_WSIGMETER(widget), 0, 10, 
															GTK_WSIGMETER(widget)->sig_level_meter_on | 
															GTK_WSIGMETER(widget)->sig_level_text_on );
		req->height += calc_size( GTK_WSIGMETER(widget), 0, 10, 
															GTK_WSIGMETER(widget)->noise_level_meter_on | 
															GTK_WSIGMETER(widget)->noise_level_text_on );
	}
	else{
		req->height= WSIGMETER_DEFAULT_HEIGHT;
	}
}
static void 
gtk_wsigmeter_size_allocate( GtkWidget *widget,
														 GtkAllocation *alloc )
{
	GtkwSigMeter *sig;
	g_return_if_fail (widget != NULL);
	g_return_if_fail (GTK_IS_WSIGMETER (widget));
	g_return_if_fail (alloc != NULL);

	widget->allocation = *alloc;
	if( GTK_WIDGET_REALIZED(widget) ){
		sig = GTK_WSIGMETER( widget );
		gdk_window_move_resize( widget->window,
														alloc->x, alloc->y,
														alloc->width, alloc->height );
		/*
		 * logic updates
		 */
		gtk_wsigmeter_configure( widget, NULL );
	}
}
static gint
gtk_wsigmeter_configure( GtkWidget *widget,
												 GdkEventConfigure *event )
{
	GtkwSigMeter *sig;
	g_return_val_if_fail( widget != NULL, FALSE );
	g_return_val_if_fail( GTK_IS_WSIGMETER( widget ), FALSE );
/*	g_return_val_if_fail( event != NULL, FALSE );*/
	
	sig = GTK_WSIGMETER(widget);
	if( sig->pixmap ){
		g_object_unref( sig->pixmap );
	}
	sig->pixmap = gdk_pixmap_new( widget->window,
																widget->allocation.width,
																widget->allocation.height,
																-1 );
	gdk_draw_rectangle( sig->pixmap,
									 		widget->style->bg_gc[widget->state],
											TRUE,
											0, 0,
											widget->allocation.width,
											widget->allocation.height );
	return TRUE;
}
static gint
gtk_wsigmeter_expose( GtkWidget *widget,
											GdkEventExpose *event )
{
	GtkwSigMeter *sig;
	g_return_val_if_fail( widget != NULL, FALSE );
	g_return_val_if_fail( GTK_IS_WSIGMETER( widget ), FALSE );
	g_return_val_if_fail( event != NULL, FALSE );
	
	sig = GTK_WSIGMETER(widget);
	
	gdk_draw_pixmap( widget->window,
									 widget->style->bg_gc[widget->state],
									 sig->pixmap,
									 event->area.x, event->area.y,
									 event->area.x, event->area.y,
									 event->area.width, event->area.height );
	return FALSE;
}
static void
gtk_wsigmeter_parse_wireless( GtkWidget *widget )
{
	wireless_config wc;
	iwstats stats;
	iwrange range;
	GtkwSigMeter *sig = GTK_WSIGMETER(widget);
	iw_get_basic_config( sig->skfd, sig->device_name, &wc );

	if( wc.essid_on == 0 ){
		sig->network_on = FALSE;
		return;
	}
	sig->network_on = TRUE;
/*	printf( "wireless protocol: %s\n", wc.name );
	printf( "wireless freq: %f\n", wc.freq / 1000000000 );
	printf( "wireless network: %s\n", wc.essid );*/
	iw_get_stats( sig->skfd, sig->device_name, &stats );
	iw_get_range_info( sig->skfd, sig->device_name, &range );

/*	printf( "link quality: %u/%u, signal level: %u/%u, noise level: %u/%u\n", 
			stats.qual.qual, range.max_qual.qual, 
			stats.qual.level, range.max_qual.level, 
			stats.qual.noise, range.max_qual.noise );*/
	if( stats.qual.updated ){
		sig->link_quality = stats.qual.qual;
		sig->max_link_quality = range.max_qual.qual;
		sig->sig_level = stats.qual.level;
	}
	sig->max_sig_level = range.max_qual.level;
	sig->noise_level = stats.qual.noise;
	sig->max_noise_level = range.max_qual.noise;
	
	sig->mode = GTK_WIRELESS_INFRA;
	
	sig->freq = wc.freq;
	if( wc.has_essid )
		strcpy( sig->network_id, wc.essid );
	strcpy( sig->protocol, wc.name );
	
/*
	struct iwreq pwrq;
	if( iw_get_ext( sig->skfd, sig->device_name, SIOCGIWNICKN, &pwrq ) == -1 ){
		printf( "nickname not provided\n" );
		return;
	}
	printf( "executed command 1\n" );
	printf( "length: %d\n", pwrq.u.essid.length );
	if( pwrq.u.data.length > 0 ){
		char *buffer = (char*)malloc( sizeof(char)*(pwrq.u.essid.length + 1) );
		strncpy( buffer, pwrq.u.essid.pointer, pwrq.u.essid.length );
		buffer[pwrq.u.essid.length] = '\0';
		printf( "executed command 2\n" );
		printf( "%s\n", buffer );
		free( buffer );
	}*/
}
static void set_color( GdkColor *color, gint strength )
{
	gfloat s = (gfloat)strength;
	s /= 100;
	color->pixel = 1;
	if( strength >= 80 ){
		color->red   = 2000;
		color->green = 65535;
		color->blue  = 45535;
	}
	else if( strength > 50 && strength < 80 ){
		color->red = 65535;
		color->green = 26000;
		color->blue = 4000;
	}
	else if( strength <= 50 ){
		color->pixel = 1;
		color->red = 65535;
		color->green = 2000;
		color->blue = 535;
	}
	else{
		color->red = 1;
		color->green = 1;
		color->blue = 1;
	}
/*		printf( "strength: %d,  red: %d, green: %d, blue: %d\n", 
						strength, color->red, color->green, color->blue );*/
}

// return's offset y value 
static gint 
draw_values( GtkwSigMeter *sig, GdkGC *gc,
						 gint x, gint y, gint barwidth, 
						 gint offset, const char *text, 
						 gboolean meter_on, gboolean text_on )
{
	gint draw_y = y;
	gint text_width, text_height;

	pango_layout_set_text( sig->layout, text, strlen( text ) );
	pango_layout_get_pixel_size( sig->layout, &text_width, &text_height );
	/* draw the visual indicator */
	if( meter_on ){
		gdk_draw_rectangle( sig->pixmap, gc, TRUE, x, draw_y, barwidth, text_height );
		gdk_draw_rectangle( sig->pixmap, GTK_WIDGET(sig)->style->dark_gc[GTK_WIDGET(sig)->state], 
				FALSE, x, draw_y, GTK_WIDGET(sig)->allocation.width - (2*offset), text_height );
	}
	/* show the percentage */
	if( text_on ){
		gdk_draw_layout( sig->pixmap,
										 GTK_WIDGET(sig)->style->text_gc[GTK_WIDGET(sig)->state], 
										 x, 
										 draw_y,
										 sig->layout );
	}
	if( meter_on || text_on ){
		draw_y += text_height + offset;
	}
	return draw_y;
}
static gboolean
gtk_wsigmeter_update( GtkWidget *widget )
{
	GdkColor fcolor;
	GdkRectangle update_rect;
	gint x, y, draw_y, width, height, offset = 10;
	gfloat percentage;
	gint strength;
	gint barwidth;
	gint text_width, text_height;
	static char text[256];
	GtkwSigMeter *sig = GTK_WSIGMETER(widget);
	GdkGC *gc;

	gtk_wsigmeter_parse_wireless( widget );
	/*
	 * get the width of the pixmap
	 * add offset to x and y to pad the
	 * sig meter
	 * calculate the precentage of signal strength
	 * multiply the precentage by 100 to get
	 * the width of the meter
	 * and finally draw to the screen
	 */

	/* get widget dimensions */
	width 		 = widget->allocation.width - 2*offset;
	height 		 = widget->allocation.height - offset;
	x 				 = widget->allocation.x + offset;
	draw_y = y = widget->allocation.y;
	/* clear the widget */
	
	gdk_draw_rectangle( sig->pixmap, widget->style->bg_gc[widget->state],
			TRUE, 0, 0,
			widget->allocation.width, widget->allocation.height );

	if( sig->network_on == FALSE ){
	//	g_object_unref( gc );
		update_rect.x 		 = widget->allocation.x;
		update_rect.y 		 = widget->allocation.y;
		update_rect.width  = widget->allocation.width;
		update_rect.height = widget->allocation.height;
		snprintf( text, 256, "Network Not Available: %s", sig->device_name );
		pango_layout_set_text( sig->layout, text, strlen( text ) );
		pango_layout_get_pixel_size( sig->layout, &text_width, &text_height );
		gdk_draw_layout( sig->pixmap,
										 widget->style->text_gc[widget->state], 
										 x,
										 y,
										 sig->layout );
		gtk_widget_draw( widget, &update_rect );
		return TRUE;
	} 
	gc = gdk_gc_new( sig->pixmap ); 
	/* Write the Network essid */
	if( sig->network_info_on ){
		snprintf( text, 256, "Network: %s, Frequency: %.3fGhz", sig->network_id, ((gfloat)sig->freq) / 1000000000.0 );
		pango_layout_set_text( sig->layout, text, strlen( text ) );
		pango_layout_get_pixel_size( sig->layout, &text_width, &text_height );
		gdk_draw_layout( sig->pixmap,
										 widget->style->text_gc[widget->state], 
										 x,
										 y,
										 sig->layout );
	}
	/* display the link quality
	 * calculate the link quality */
	percentage = (gfloat)sig->link_quality / (gfloat)sig->max_link_quality;
	strength = percentage * 100;
	barwidth = width*percentage;

	/* set color based on quality level */
	set_color( &fcolor, strength );
	gdk_gc_set_rgb_fg_color( gc, &fcolor );

	snprintf( text, 256,"Link Quality: %u/%u, %d\%%", sig->link_quality, sig->max_link_quality, strength );
	/* draw the link quality */
	draw_y = draw_values( sig, gc,
						 x, draw_y+text_height, barwidth, 
						 offset, text, 
						 sig->link_qual_meter_on, 
						 sig->link_qual_text_on );
	/* display the signal level */

	percentage = MIN( sig->sig_level *0.3, sig->max_sig_level ) / 100.0; /* from wmwave version 0.4 @ http://packages.debian.org/testing/x11/wmwave.html*/
	strength 	 = percentage * 100;
	barwidth   = width*percentage;
	/* set color based on quality level */
	set_color( &fcolor, strength );
	gdk_gc_set_rgb_fg_color( gc, &fcolor );
	/* printf( "sig level: %.4f, max: %u, percent: %.4f\n", sig->sig_level*0.3, sig->max_sig_level, percentage );*/
	snprintf( text, 256, "Signal Level: %.4f/%u, %d\%%", 
			(float)(sig->sig_level*0.3), sig->max_sig_level, strength );
	draw_y = draw_values( sig, gc,
						 x, draw_y, barwidth, 
						 offset, text, 
						 sig->sig_level_meter_on, 
						 sig->sig_level_text_on );
	/* display the noise level */
	percentage = MIN( sig->noise_level *0.3, sig->max_noise_level) / 100.0; /* from wmwave version 0.4 @ http://packages.debian.org/testing/x11/wmwave.html*/
	strength = percentage * 100;
	barwidth = width*percentage;
	/* set color based on quality level */
	set_color( &fcolor, strength );
	gdk_gc_set_rgb_fg_color( gc, &fcolor );
	
	snprintf( text, 256, "Noise Level: %u/%u , %d\%%", sig->noise_level, sig->max_noise_level, strength );
	text[255] = 0; /* paranoia */
	draw_y = draw_values( sig, gc,
						 x, draw_y, barwidth, 
						 offset, text, 
						 sig->noise_level_meter_on, 
						 sig->noise_level_text_on );
/*	gdk_draw_rectangle( sig->pixmap, GTK_WIDGET(sig)->style->dark_gc[GTK_WIDGET(sig)->state], 
			FALSE, widget->allocation.x+2, widget->allocation.y+2, 
			widget->allocation.width-6, widget->allocation.height-6 );*/
	
	/* cleanup */

	g_object_unref( gc );
	update_rect.x = widget->allocation.x;
	update_rect.y = widget->allocation.y;
	update_rect.width = widget->allocation.width;
	update_rect.height = widget->allocation.height;
	gtk_widget_draw( widget, &update_rect );
	return TRUE;
}
