/* gtkwsigmeter.h
 * Copyright (C) 2002 Todd Fisher
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __GTK_WSIGMETER_H__
#define __GTK_WSIGMETER_H__

#include <gdk/gdk.h>
#include <gtk/gtkadjustment.h>
#include <gtk/gtkwidget.h>


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define GTK_WSIGMETER(obj)          GTK_CHECK_CAST (obj, gtk_wsigmeter_get_type (), GtkwSigMeter)
#define GTK_WSIGMETER_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtk_wsigmeter_get_type (), GtkwSigMeterClass)
#define GTK_IS_WSIGMETER(obj)       GTK_CHECK_TYPE (obj, gtk_wsigmeter_get_type ())


typedef struct _GtkwSigMeter        GtkwSigMeter;
typedef struct _GtkwSigMeterClass   GtkwSigMeterClass;
typedef enum _GtkWirelessMode{
	GTK_WIRELESS_INFRA, GTK_WIRELESS_AD_HOC
}GtkWirelessMode;
struct _GtkwSigMeter
{
  GtkWidget widget;
	gint timer;
	gint update_interval;
	/* The Link Quality as given by the iwconfig tool */
	gint link_quality, max_link_quality;
	gint sig_level, max_sig_level;
	/* The amount of noise  mines -102 dBm */
	gint noise_level, max_noise_level;
	/* wireless frequency 2.4Ghz */
	gfloat freq;
	/* Can be infra or AD-HOC */
	GtkWirelessMode mode;
	/* device name e.g. eth0 */
	gchar *device_name;
	/* the network ESSID id */
	gchar network_id[256];
	/* wireless protocol type */
	gchar protocol[256];
	/* the following are not implemented yet 7/14/2003 */
	/* nick name */
	gchar nickname[256]; /* not sure what this is?? */
	/* hardware address */
	gchar hwaddr[256];
	/* ip address */
	gchar addr[256];

	/* internal use only */
	GdkPixmap *pixmap;
	PangoLayout *layout;
	int skfd; /* socket to wireless device */
	gboolean link_qual_text_on, link_qual_meter_on;
	gboolean sig_level_text_on, sig_level_meter_on;
	gboolean noise_level_text_on, noise_level_meter_on;
	gboolean network_info_on;
	gboolean network_on;
};

struct _GtkwSigMeterClass
{
  GtkWidgetClass parent_class;
};


GtkWidget*      gtk_wsigmeter_new                         (void);
GtkType         gtk_wsigmeter_get_type                    (void);
void            gtk_wsigmeter_set_update_interval         (GtkwSigMeter*, gint update_interval );
gint            gtk_wsigmeter_get_update_interval         (GtkwSigMeter* );
gint            gtk_wsigmeter_get_link_quality            (GtkwSigMeter*, gint *max );
gint            gtk_wsigmeter_get_sig_level               (GtkwSigMeter*, gint *max );
gint            gtk_wsigmeter_get_noise_level             (GtkwSigMeter*, gint *max );
gint            gtk_wsigmeter_get_freq		                (GtkwSigMeter* );
GtkWirelessMode gtk_wsigmeter_get_mode                    (GtkwSigMeter* );
const gchar*    gtk_wsigmeter_get_network_id              (GtkwSigMeter* );
const gchar*    gtk_wsigmeter_get_protocol                (GtkwSigMeter* );
const gchar*		gtk_wsigmeter_get_device_name							(GtkwSigMeter* );
void						gtk_wsigmeter_set_device_name							(GtkwSigMeter*, const gchar *name );
const gchar*    gtk_wsigmeter_get_nickname                (GtkwSigMeter* );
const gchar*    gtk_wsigmeter_get_hwaddr                  (GtkwSigMeter* );
/* control what information the widget display's */
void						gtk_wsigmeter_toggle_link_qual_text       (GtkwSigMeter*, gboolean on );
void						gtk_wsigmeter_toggle_link_qual_meter      (GtkwSigMeter*, gboolean on );
void						gtk_wsigmeter_toggle_signal_level_text    (GtkwSigMeter*, gboolean on );
void						gtk_wsigmeter_toggle_signal_level_meter   (GtkwSigMeter*, gboolean on );
void						gtk_wsigmeter_toggle_noise_level_text     (GtkwSigMeter*, gboolean on );
void						gtk_wsigmeter_toggle_noise_level_meter    (GtkwSigMeter*, gboolean on );
void						gtk_wsigmeter_toggle_network_info         (GtkwSigMeter*, gboolean on );

#define gtk_wsigmeter_get_link_qual_text_state( sigmeter )  GTK_WSIGMETER(sigmeter)->link_qual_text_on
#define gtk_wsigmeter_get_link_qual_meter_state( sigmeter ) GTK_WSIGMETER(sigmeter)->link_qual_meter_on
#define gtk_wsigmeter_get_sig_level_text_state( sigmeter ) GTK_WSIGMETER(sigmeter)->sig_level_text_on
#define gtk_wsigmeter_get_sig_level_meter_state( sigmeter ) GTK_WSIGMETER(sigmeter)->sig_level_meter_on
#define gtk_wsigmeter_get_noise_level_text_state( sigmeter ) GTK_WSIGMETER(sigmeter)->noise_level_text_on
#define gtk_wsigmeter_get_noise_level_meter_state( sigmeter ) GTK_WSIGMETER(sigmeter)->noise_level_meter_on
#define gtk_wsigmeter_get_network_info_state( sigmeter ) GTK_WSIGMETER(sigmeter)->network_info_on

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __GTK_WSIGMETER_H__ */
