/* wsm.c
 * Copyright (C) 2002 Todd Fisher
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk/gtk.h>
#include "gtkwsigmeter.h"

typedef struct ToggleId{
	int id;
	GtkWidget *meter;
	GtkWidget *toggle;
	const char *label;
}ToggleId;

typedef struct WidgetArg{
	GtkWidget *widgets[10];
}WidgetArg;

gint 
quit_app( gpointer null )
{
	gtk_main_quit();
	return FALSE;
}
void 
toggle_func( GtkToggleButton *togglebutton,
						 gpointer ptr )
{
	ToggleId *toggle_id = (ToggleId*)ptr;
	switch( toggle_id->id ){
	case 0: /* link quality meter on/off */
		gtk_wsigmeter_toggle_link_qual_meter( GTK_WSIGMETER(toggle_id->meter),
				!gtk_wsigmeter_get_link_qual_meter_state( GTK_WSIGMETER( toggle_id->meter ) ) );
		break;
	case 1: /* link quality text on/off */
		gtk_wsigmeter_toggle_link_qual_text( GTK_WSIGMETER(toggle_id->meter),
				!gtk_wsigmeter_get_link_qual_text_state( GTK_WSIGMETER( toggle_id->meter ) ) );
		break;
	case 2: /* signal level meter on/off */
		gtk_wsigmeter_toggle_signal_level_meter( GTK_WSIGMETER(toggle_id->meter),
				!gtk_wsigmeter_get_sig_level_meter_state( GTK_WSIGMETER( toggle_id->meter ) ) );
		break;
	case 3: /* signal level text on/off */
		gtk_wsigmeter_toggle_signal_level_text( GTK_WSIGMETER(toggle_id->meter),
				!gtk_wsigmeter_get_sig_level_text_state( GTK_WSIGMETER( toggle_id->meter ) ) );
		break;
	case 4: /* noise level meter on/off */
		gtk_wsigmeter_toggle_noise_level_meter( GTK_WSIGMETER(toggle_id->meter),
				!gtk_wsigmeter_get_noise_level_meter_state( GTK_WSIGMETER( toggle_id->meter ) ) );
		break;
	case 5: /* noise level text on/off */
		gtk_wsigmeter_toggle_noise_level_text( GTK_WSIGMETER(toggle_id->meter),
				!gtk_wsigmeter_get_noise_level_text_state( GTK_WSIGMETER( toggle_id->meter ) ) );
		break;
	case 6: /* essid & freq on/off */
		gtk_wsigmeter_toggle_network_info( GTK_WSIGMETER(toggle_id->meter),
				!gtk_wsigmeter_get_network_info_state( GTK_WSIGMETER(toggle_id->meter) ) );
		break;
	default:
		break;
	}
}
void set_timeout( GtkWidget *button, gpointer wp )
{
	GtkWidget *meter = ((WidgetArg*)wp)->widgets[0];
	GtkWidget *spin  = ((WidgetArg*)wp)->widgets[1];
	gtk_wsigmeter_set_update_interval( GTK_WSIGMETER(meter), 
			gtk_spin_button_get_value( GTK_SPIN_BUTTON(spin) ) * 200 );
}
void 
opt_show_click( GtkWidget *button, gpointer wp )
{
	GtkWidget *opt_panel = ((WidgetArg*)wp)->widgets[2];
	GtkWidget *opt_indicator = ((WidgetArg*)wp)->widgets[3];
	GtkWidget *window = ((WidgetArg*)wp)->widgets[4];
	static gint height = -1;
	gint width, blank = -1;

	if( !GTK_WIDGET_VISIBLE( opt_panel ) ){
		if( height == -1 ){
			gtk_window_get_size( GTK_WINDOW( window ), &width, &height );
		}
		gtk_widget_show_all( opt_panel );
		gtk_arrow_set( GTK_ARROW(opt_indicator), GTK_ARROW_DOWN, GTK_SHADOW_NONE );
	}
	else{
		gtk_window_get_size( GTK_WINDOW( window ), &width, &blank );
		gtk_widget_hide_all( opt_panel );
		gtk_arrow_set( GTK_ARROW(opt_indicator), GTK_ARROW_RIGHT, GTK_SHADOW_NONE );
		gtk_window_resize( GTK_WINDOW(window), width, height );
	}
}
static void set_device_name( GtkWidget *widget, GtkWidget *meter )
{
	gtk_wsigmeter_set_device_name( GTK_WSIGMETER(meter), gtk_entry_get_text( GTK_ENTRY(widget) ) );
}
int main( int argc, char **argv )
{
	GtkWidget *window;
	GtkWidget *meter;
	GtkWidget *vbox, *hbox;
	GtkWidget *spin_button;
	GtkObject *spin_adjuster;
	GtkWidget *timeout_set;
	GtkWidget *opt_show;
	GtkWidget *opt_panel;
	GtkWidget *opt_indicator;
	GtkWidget *opt_device_entry;

	int i;
	WidgetArg widget_args;
	ToggleId ids[7] = { {0,NULL, NULL, "Link Quality Meter"},
											{1,NULL, NULL, "Link Quality Text"},
											{2,NULL, NULL, "Signal Level Meter"},
											{3,NULL, NULL, "Signal Level Text"},
											{4,NULL, NULL, "Noise  Level Meter"},
											{5,NULL, NULL, "Noise  Level Text"},
											{6,NULL, NULL, "Network Information"}
										};
	
	gtk_init( &argc, &argv );

	/* Create the window */
	window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_window_set_title( GTK_WINDOW(window), "Wireless Signal Meter" );
	//gtk_window_set_resizable (GTK_WINDOW(window), FALSE);
	opt_panel = gtk_vbox_new( FALSE, 0 );
	opt_show  = gtk_button_new();
	opt_indicator = gtk_arrow_new( GTK_ARROW_RIGHT, GTK_SHADOW_NONE );
	gtk_container_add( GTK_CONTAINER( opt_show ), opt_indicator );
	gtk_button_set_relief( GTK_BUTTON(opt_show), GTK_RELIEF_NONE );
	vbox = gtk_vbox_new( FALSE, 5 );
	meter = gtk_wsigmeter_new();
	hbox = gtk_hbox_new( FALSE, 5 );
	/* set up spin button for timeout */
	spin_adjuster = gtk_adjustment_new( 1, 1, 120, 1, 20, 20 );
	spin_button 	= gtk_spin_button_new( GTK_ADJUSTMENT( spin_adjuster ), 0.5, 1 );
	timeout_set		= gtk_button_new_with_label( "Set Time Out:" );
	
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(meter), FALSE, FALSE, 0 );
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(opt_show), FALSE, FALSE, 0 );
	gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(opt_panel), FALSE, FALSE, 0 );
	gtk_container_add( GTK_CONTAINER(window), vbox );

	/* set up toggle button options */
	for( i = 0; i < 7; ++i ){
		ids[i].toggle = gtk_check_button_new_with_label( ids[i].label );
		ids[i].meter = meter;
		gtk_box_pack_start( GTK_BOX(opt_panel), ids[i].toggle, FALSE, FALSE, 0 );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(ids[i].toggle), TRUE );
		g_signal_connect( G_OBJECT(ids[i].toggle), "toggled", 
											GTK_SIGNAL_FUNC(toggle_func), &(ids[i]) );
		gtk_widget_show( ids[i].toggle );
	}
	gtk_box_pack_start( GTK_BOX(opt_panel), hbox, FALSE, FALSE, 5 );
	gtk_box_pack_start( GTK_BOX(hbox), timeout_set, FALSE, FALSE, 5 );
	gtk_box_pack_start( GTK_BOX(hbox), spin_button, FALSE, FALSE, 5 );
	opt_device_entry = gtk_entry_new();
	gtk_entry_set_text( GTK_ENTRY( opt_device_entry ), 
											gtk_wsigmeter_get_device_name( GTK_WSIGMETER(meter) ) );
	gtk_box_pack_start( GTK_BOX(hbox), opt_device_entry, FALSE, FALSE, 5 );
	g_signal_connect( G_OBJECT( opt_device_entry ), "activate", 
										G_CALLBACK(set_device_name),meter );
	/* set up signal handlers */
	g_signal_connect( G_OBJECT(window), "delete-event",
										G_CALLBACK(quit_app),NULL );
	widget_args.widgets[0]  = meter;
	widget_args.widgets[1] = spin_button;
	g_signal_connect( G_OBJECT(timeout_set), "clicked",
										G_CALLBACK(set_timeout), (gpointer)&widget_args );
	widget_args.widgets[2] = opt_panel;
	widget_args.widgets[3] = opt_indicator;
	widget_args.widgets[4] = window;
	g_signal_connect( G_OBJECT(opt_show), "clicked",
										G_CALLBACK(opt_show_click), (gpointer)&widget_args ); 
	/* show all widgets */
	gtk_widget_show( meter );
	gtk_widget_show( opt_show );
	gtk_widget_show( opt_indicator );
/*	gtk_widget_show( hbox );
	gtk_widget_show( spin_button );
	gtk_widget_show( timeout_set );*/
	gtk_widget_show( vbox );
	gtk_widget_show( window );

	gtk_main();
	return 0;
}
